import pytest
from .hashing_search import hashing_search
from .linear_search import linear_search


@pytest.mark.parametrize("a, b", [('aaa', 'aaa'),
                                ('abcdefg', 'abcdefg'),
                                ('abcd', 'abcd'),
                                ('1111', '1111')])
def test_string_compare_true(a:str, b:str):
    assert hashing_search(a, b) == True
    assert linear_search(a, b) == True


@pytest.mark.parametrize('a, b', [('abcd', 'bcda'),
                                    ('qwer', 'sdfw'),
                                    ('dfgdgfd', 'df'),
                                    ('12334', '12344')])
def test_string_compare_false(a:str, b:str):
    assert hashing_search(a, b) == False
    assert linear_search(a, b) == False



import csv
from timeit import Timer
import os
import shutil
import time
import statistics


def create_dir():
    if os.path.exists("load_testing_measurements"):
        shutil.rmtree("load_testing_measurements")
    os.mkdir("load_testing_measurements")


def time_of_program(func, arg1, arg2):
    t = Timer("""{0}""".format(func(arg1, arg2)), setup="""x = range(1000)""")
    time = t.timeit()
    return round(time, 5)


def measure_time(func):
    create_dir()

    lst = os.listdir(path='load_testing_data/')

    lst.sort()
    elem = 0
    data_sizes_qt = 0  # кол-во размеров данных

    sizes = []

    for i in range(len(lst)):

        if lst[i].split('_')[0] != elem:
            sizes.append(int(lst[i].split('_')[0]))
            data_sizes_qt += 1
            elem = lst[i].split('_')[0]

    count = len(lst) // data_sizes_qt  # кол-во наборов данных опр.размера

    sizes.sort()

    start = int(sizes[0])

    end = int(sizes[-1]) + 1

    step = int(sizes[1]) - int(sizes[0])

    times = [0] * data_sizes_qt

    for i in range(data_sizes_qt):
        times[i] = [0] * count

    for m in range(start, end, step):

        for i in range(count):
            with open('load_testing_data/{0}_{1}.txt'.format(m, i), 'r') as f:
                linelist = (f.readlines())

    for m in range(data_sizes_qt):

        for i in range(count):
            times[m][i] = (time_of_program(func, linelist[0][:-1:], linelist[1]))

    timestamp = int(time.time())
    filename = "%s.csv" % timestamp

    data_sizes = len(range(start, end, step))

    with open('load_testing_measurements/{0}'.format(filename), 'w') as csvfile:
        filewriter = csv.writer(csvfile, delimiter=' ',
                                quotechar=' ', quoting=csv.QUOTE_MINIMAL)
        filewriter.writerow(['size', 'min', 'max', 'avg', 'median'])
        size = start
        for i in range(data_sizes):
            times[i].sort()
            elem = times[i]

            mn = min(elem)
            mx = max(elem)
            avg = round(sum(elem) / len(elem), 5)

            md = round(statistics.median(elem), 5)

            filewriter.writerow([size, mn, mx, avg, md])

            size += step



import os
import shutil
import random


def create_dir():
    if os.path.exists("load_testing_data"):
        shutil.rmtree("load_testing_data")
    os.mkdir("load_testing_data")


def generate_data(start: int, end: int, step: int, count: int):
    create_dir()

    def tests_generate(size, count):

        def string_generation(size) -> str:
            string = ''
            for i in range(size):
                string += chr(random.randint(97, 122))
            return string

        for i in range(count):
            string1 = string_generation(size)
            string2 = ''
            if random.randint(0, 1):
                string2 = string1
            else:
                string2 = string_generation(size)

            with open('load_testing_data/{0}_{1}.txt'.format(size, i), 'w') as f:
                f.write(string1 + '\n')
                f.write(string2)

    for i in range(start, end, step):
        tests_generate(i, count)



def hashing_search(fstring: str, sstring: str) -> bool:
    def factors_generation(string: str) -> list:
        factors = [1]

        for i in range(1, len(string)):
            factor = factors[i - 1] * 2
            factors.append(factor)
        return factors

    def hash_generation(string, degree: list) -> list:
        hashs = [ord(string[0])]

        for i in range(1, len(fstring)):
            symb_hash = hashs[i - 1] + degree[i] * ord(string[i])
            hashs.append(symb_hash)
        return hashs

    if len(fstring) == len(sstring) and len(fstring) > 0:
        factors = factors_generation(sstring)
        fhashs = hash_generation(fstring, factors)
        shashs = hash_generation(sstring, factors)
        if fhashs[-1] == shashs[-1]:
            return True
        else:
            return False
    else:
        return False



def linear_search(fstring: str, sstring: str) -> bool:
    if len(fstring) == len(sstring):
        fstring = list(fstring)
        sstring = list(sstring)

        for i in range(len(fstring)):
            if fstring[i] != sstring[i]:
                return False

        return True

    else:
        return False



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['stringcompare', 'stringcompare.functions']

package_data = \
{'': ['*']}

install_requires = \
['click>=8.0.1,<9.0.0', 'matplotlib>=3.4.2,<4.0.0']

entry_points = \
{'console_scripts': ['stringcompare = stringcompare.stringcompare:main']}

setup_kwargs = {
    'name': 'stringcompare',
    'version': '0.1.9',
    'description': 'This package contains functions for comparing strings, finding substrings (the first occurrence of the desired string) ',
    'long_description': None,
    'author': 'Marsel Muzafarov, Ravil Nurgaliev',
    'author_email': None,
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)



from stringcompare.functions import hashing_search
from stringcompare.functions import linear_search
from stringcompare.functions import create_chart
from stringcompare.functions import generate_data
from stringcompare.functions import measure_time

import click


@click.group()
def main():
    pass


@main.command()
@click.option('--data', nargs=2, help='enter 2 strings')
def process1(data):
    data1 = data[0]
    data2 = data[1]
    print(linear_search(data1, data2))


@main.command()
@click.option('--data', nargs=2, help='enter 2 strings')
def process2(data):
    data1 = data[0]
    data2 = data[1]
    print(hashing_search(data1, data2))


@main.command()
@click.option('--start', help='enter START value')
@click.option('--end', help='enter END value')
@click.option('--step', help='enter STEP value')
@click.option('--count', help='enter COUNT value')
def generate(start, end, step, count):
    generate_data(int(start), int(end), int(step), int(count))


@main.command()
@click.option('--func', help='enter func name: linear_search or hashing_search')
def measure_algo(func):
    measure_time(func)


@main.command()
@click.option('--file', help='enter path to csv table')
def chart(file: str):
    create_chart(file)


if __name__ == '__main__':
    main()



import os
import csv
import matplotlib.pyplot as plt
import shutil
import time


def csv_reader(file_obj):
    reader = csv.reader(file_obj)
    for row in reader:
        print(" ".join(row))


def create_dir():
    if os.path.exists("load_testing_plots"):
        shutil.rmtree("load_testing_plots")
    os.mkdir("load_testing_plots")


def create_chart(csv_file_path: str):
    path = csv_file_path

    sizes = []

    maxs = []

    mins = []

    medians = []

    avgs = []

    with open(path, encoding='utf-8') as r_file:
        file_reader = csv.DictReader(r_file, delimiter=" ")

        count = 0

        for row in file_reader:
            sizes.append(row["size"])

            mins.append(row["min"])

            maxs.append(row["max"])

            avgs.append(row["avg"])

            medians.append(row["median"])

            count += 1

    mins.sort()
    maxs.sort()
    medians.sort()
    avgs.sort()

    plt.figure(figsize=(12, 7))
    plt.subplot(221)
    plt.plot((sizes), (mins), '-', label='minimal')
    plt.legend()
    plt.subplot(222)
    plt.plot((sizes), (maxs), '--', label='maximum')
    plt.legend()
    plt.subplot(223)
    plt.plot((sizes), (medians), '-.', label='median')
    plt.legend()
    plt.subplot(224)
    plt.plot((sizes), (avgs), ':', label='average')
    plt.legend()
    timestamp = int(time.time())

    create_dir()

    plt.savefig('load_testing_plots/{0}'.format(timestamp))
